if (!('contains' in String.prototype)) {
  String.prototype.contains = function (str, startIndex) {
    return ''.indexOf.call(this, str, startIndex) !== -1;
  };
}

if (typeof String.prototype.startsWith != 'function') {
  String.prototype.startsWith = function (prefix) {
    return this.slice(0, prefix.length) == prefix;
  };
}

if (typeof String.prototype.endsWith != 'function') {
  String.prototype.endsWith = function (suffix) {
    return this.slice(-suffix.length) == suffix;
  };
}

var ngp = require('./ecxngp');

var funcStepsGet = [
  {
    continue: 'next',
    name: 'get_profilesInUse',
    verb: 'GET',
    path: null,
    ip: null,
    port: null,
    type: null,
    func: null,
    embedOptions: null,
    pageSize: null,
    pageStartIndex: null,
    cb: function (res, data) {
      logMsg('info', 'Step=' + this.name + ' finished');
    },
  },
  {
    continue: 'next',
    name: 'get_BackupSLA',
    verb: 'GET',
    path: null,
    ip: null,
    port: null,
    type: null,
    func: null,
    embedOptions: null,
    pageSize: null,
    pageStartIndex: null,
    cb: function (res, data) {
      logMsg('info', 'Step=' + this.name + ' finished');
    },
  },
];

const getBackupSLAsStepFuncs = {
  get_profilesInUse: function (step, funcStream, ecxSessionId, oFromNg, cb) {
    getProfilesInUse(step, funcStream, ecxSessionId, oFromNg, cb);
  },
  get_BackupSLA: function (step, funcStream, ecxSessionId, oFromNg, cb) {
    genBackupSLAs(step, funcStream, ecxSessionId, oFromNg, cb);
  },
};

exports.init = init;

function init(req, res, subType, embedSLAOptions, pageSize, pageStartIndex) {
  var ngState = ngp.getSessionState(req, res);

  if (ngState.stale) return;

  var funcSteps = null;
  var genericFuncName = null;

  if (req.method == 'GET') {
    genericFuncName = 'get_BackupSLAPolicy';
    funcSteps = JSON.parse(JSON.stringify(funcStepsGet));
  }

  var ecxSid = ngState.ecxSid;
  res.ecxSid = ecxSid;

  var stepsStream = JSON.parse(JSON.stringify(funcSteps));

  stepsStream.forEach(function (step) {
    step.ip = ngp.ecxSidHash[ecxSid].host;
    step.port = ngp.ecxExternal;

    if (step.name == 'get_BackupSLA') step.embedOptions = embedSLAOptions;
    step.pageSize = pageSize;

    step.pageStartIndex = pageStartIndex;

    step.func = getBackupSLAsStepFuncs[step.name];

    step.cb = function (ecxSid, data) {
      ngp.logMsg(
        'info',
        "Finished step '" +
          ngp.getCurrentFunc(step.reqId, ecxSid) +
          '(' +
          this.name +
          ")' in session ID '" +
          ecxSid +
          "'",
      );

      if (data) {
        ngp.logMsg(
          'debug',
          "Data returned by step '" +
            ngState.currentFunc +
            '(' +
            this.name +
            ")' in session ID '" +
            ecxSid +
            "'=" +
            JSON.stringify(data),
        );
      }
    };
  });

  ngp.createRequestStore(stepsStream, ecxSid, req.reqId, genericFuncName);

  ngState.currentFunc = genericFuncName;
  ngp.setRequestState(req.reqId, ecxSid, 'statusSpec', subType);
  return stepsStream;
}

var genBackupSLAs = function (step, funcStream, ecxSessionId, oFromNg, cb) {
  var subType = ngp.getRequestState(step.reqId, ecxSessionId, 'statusSpec');

  if (step.pageStartIndex)
    var path =
      '/api/endeavour/job?excludeMarkDeleted=true&pageStartIndex=' +
      step.pageStartIndex +
      '&pageSize=' +
      step.pageSize +
      '&filter=';
  else var path = '/api/endeavour/job?excludeMarkDeleted=true&pageSize=' + step.pageSize + '&filter=';

  var profilesInUse = ngp.getRequestState(step.reqId, ecxSessionId, 'profilesInUse');
  if (profilesInUse && profilesInUse.length == 0) {
    ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, { statusCode: 200, response: null });
    return;
  }

  const getProfilesFilter = function() {
    if (profilesInUse && profilesInUse !== '*')
      return '{"property":"name","value":["' +
        profilesInUse.map(p => ngp.ngpUtils.getJobNamePrefix(subType) + "_" + p).join('","') +
        '"],"op":"IN"},';
    else
      return '{"property":"name","value":"' +ngp.ngpUtils.getJobNamePrefix(subType) +'_*"},';
  }
  const filter = "[" + getProfilesFilter() + '{"property":"type","value":"protection"}]';
  const finalPath = path + ngp.escape(filter);

  ngp.sendRequestToEcx('GET', ngp.ecxExternal, finalPath, ecxSessionId, null, function (jobO) {
    if (jobO.e || jobO.response.statusCode != 200) {
      var failResult = ngp.getGenericEcxErrorResponse(jobO, ecxSessionId, step);
      ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
    } else {
      var sprofs = jobO.rO;
      var slaPoliciesFinalArray = [];
      var sprofClones = JSON.parse(JSON.stringify(sprofs.jobs));

      var SLAsFilter = sprofClones
        .map((sprof) => '"' + sprof.name.substring(sprof.name.indexOf('_') + 1) + '"')
        .join(',');

      ngp.ngpCoreHelper.getPolicies(ecxSessionId, function (getPoliciesResObj) {
        var policiesArray = getPoliciesResObj.rO['policies'];

        var path = '/api/spec/storageprofile?noregex=true&filter=';
        var filter = '[{"property":"name","op":"IN","value":[' + SLAsFilter + ']}]';
        var escFilter = ngp.escape(filter);
        var finalPath = path + escFilter;

        ngp.sendRequestToEcx('GET', ngp.ecxExternal, finalPath, ecxSessionId, null, function (slaPolicies) {
          if (slaPolicies.e || slaPolicies.response.statusCode != 200) {
            var failResult = ngp.getGenericEcxErrorResponse(slaPolicies, ecxSessionId, step);
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
          } else {
            var slaPoliciesArray = slaPolicies.rO['storageprofiles'];

            mergeSprofToSlap(
              step,
              sprofClones,
              slaPoliciesFinalArray,
              ecxSessionId,
              policiesArray,
              slaPoliciesArray,
              'ok',
              function (rc) {
                if (rc != 'ok') {
                  var failResult = ngp.getGenericEcxErrorResponse(
                    ngp.getRequestState(step.reqId, ecxSessionId, 'addStatusErrorResponse'),
                    ecxSessionId,
                    step,
                  );
                  ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult);
                  return;
                }

                var rtrnSlapO = {
                  links: sprofs.links,
                  total: slaPoliciesFinalArray.length,
                  slapolicies: slaPoliciesFinalArray,
                };

                var successResult = {
                  statusCode: 200,
                  response: rtrnSlapO,
                };

                ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);
              },
            );
          }
        });
      });
    }
  });
};

function mergeSprofToSlap(step, sprofClones, slaPoliciesAr, ecxSid, policiesArray, slaPoliciesArray, lastRc, cb) {
  if (sprofClones.length < 1) {
    cb(lastRc);
    return;
  }

  var subType = ngp.getRequestState(step.reqId, ecxSid, 'statusSpec');
  var sprof = sprofClones.shift();
  var slaPolicyName = sprof.name.substring(sprof.name.indexOf('_') + 1);

  for (var i = 0; i < slaPoliciesArray.length; i++) {
    if (slaPoliciesArray[i].name == slaPolicyName) {
      var slapAr = [];
      var subpHash = ngp.arrayToHash(slaPoliciesArray[i].spec.subpolicy, 'type');

      if (subpHash['SNAPSHOT']) {
        slapAr.push({
          type: 'SNAPSHOT',
          retention: subpHash.SNAPSHOT.spec.option.retention, // tolerate .age or .numsnapshots
          trigger: subpHash.SNAPSHOT.trigger,
          label: subpHash.SNAPSHOT.spec.option.label,
        });
      }

      if (subpHash['SP']) {
        var slapHappy = {
          type: 'SP',
          trigger: subpHash.SP.trigger,
        };
        slapAr.push(slapHappy);
      }

      if (subpHash['REPLICATION']) {
        slapAr.push({
          type: 'REPLICATION',
          software: false,
          trigger: subpHash.REPLICATION.trigger,
        });
      }

      if (subpHash['SPPARCHIVE']) {
        slapAr.push({
          type: 'SPPARCHIVE',
          trigger: subpHash.SPPARCHIVE.trigger,
        });
      }

      if (subpHash['SPPOFFLOAD']) {
        slapAr.push({
          type: 'SPPOFFLOAD',
          trigger: subpHash.SPPOFFLOAD.trigger,
        });
      }

      if (subpHash['BACKUP']) {
        slapAr.push({
          type: 'REPLICATION',
          software: true,
          trigger: subpHash.BACKUP.trigger,
        });
      }

      var slap = {
        name: slaPolicyName,
        version: '1.0',
        spec: {
          simple: slaPoliciesArray[i].spec.simple,
          subpolicy: slapAr,
        },
        links: {},
      };

      if (slaPoliciesArray[i].spec.demo) slap.spec.demo = slaPoliciesArray[i].spec.demo;

      for (jobLinkName in sprof.links) {
        var jobLink = sprof.links[jobLinkName];
        if (jobLink.rel && (jobLink.rel == 'action' || jobLink.rel == 'self')) slap.links[jobLinkName] = jobLink;
      }

      var urlParts = slap.links.self.href.split(/\:\d{3,4}/);

      var modifiedURLParts = urlParts[0].split('//');
      var finalURLParts = modifiedURLParts[0] + '//' + ngp.ecxSidHash[ecxSid].host;

      if (ngp.productionMode == false)
        sessPath = finalURLParts + ':' + ngp.ngpPort + ngp.apiFirstStep + 'slapolicy/jobsessions?';
      else sessPath = finalURLParts + ngp.apiFirstStep + 'slapolicy/jobsessions?';

      sessPath += 'name=' + ngp.escape(slap.name) + '&subtype=' + subType;

      var jobSessLink = {
        rel: 'related',
        href: sessPath,
      };

      slap.links.jobsessions = jobSessLink;

      if (ngp.hasPropertyPath(sprof, 'lastrun.properties.statistics'.split('.')))
        slap.statistics = sprof.lastrun.properties.statistics;

      slap.status = sprof.status;
      slap.statusDisplayName = sprof.statusDisplayName;
      slap.nextFireTime = sprof.nextFireTime;
      slap.nextFireTimes = sprof.nextFireTimes;

      break;
    }
  }

  if (!slap) {
    mergeSprofToSlap(step, sprofClones, slaPoliciesAr, ecxSid, policiesArray, slaPoliciesArray, lastRc, cb);
  } else {
    addScript(step, slap, sprof, subType, ecxSid, policiesArray, function () {
      slaPoliciesAr.push(slap);
      mergeSprofToSlap(step, sprofClones, slaPoliciesAr, ecxSid, policiesArray, slaPoliciesArray, lastRc, cb);
    });
  }
}

function addScript(step, slap, sprof, subType, ecxSid, policiesArray, cb) {
  var policyName = ngp.ngpUtils.getJobNamePrefix(subType) + '_' + slap.name;
  var policyId = null;

  policiesArray.forEach(function (policy) {
    if (policyId) return;

    if (policyName == policy.name) {
      policyId = policy.id;
    }
  });

  var urlParts = slap.links.self.href.split(/\:\d{3,4}/);

  var modifiedURLParts = urlParts[0].split('//');
  var finalURLParts = modifiedURLParts[0] + '//' + ngp.ecxSidHash[ecxSid].host;

  var escSlapName = require('querystring').escape(slap.name);

  if (step.embedOptions == null || step.embedOptions == 'true') {
    if (policyId) {
      ngp.ngpCoreHelper.getPolicy(ecxSid, policyId, function (getPolicyResObj) {
        var getPolicyResponse = getPolicyResObj.rO;

        if (getPolicyResponse['script']) {
          slap.script = getPolicyResponse['script'];
        }

        if (getPolicyResponse.spec['option']) {
          slap.spec.policyOption = getPolicyResponse.spec['option'];
        }

        if (ngp.productionMode == false) {
          var applyPolicyOptionsLink = {
            rel: 'action',
            href:
              finalURLParts +
              ':' +
              ngp.ngpPort +
              ngp.apiFirstStep +
              'slapolicy?action=applyPolicyOptions&name=' +
              escSlapName +
              '&subtype=' +
              subType,
          };
        } else {
          var applyPolicyOptionsLink = {
            rel: 'action',
            href:
              finalURLParts +
              ngp.apiFirstStep +
              'slapolicy?action=applyPolicyOptions&name=' +
              escSlapName +
              '&subtype=' +
              subType,
          };
        }

        slap.links.applyPolicyOptions = applyPolicyOptionsLink;
        cb();
      });
    } else {
      cb();
    }
  } else {
    if (ngp.productionMode == false) {
      var applyOptionsLink = {
        rel: 'action',
        href:
          finalURLParts +
          ':' +
          ngp.ngpPort +
          ngp.apiFirstStep +
          'slapolicy?action=applyPolicyOptions&name=' +
          escSlapName +
          '&subtype=' +
          subType,
      };

      var optionsLink = {
        rel: 'related',
        href: finalURLParts + ':' + ngp.ecxInternal + '/api/endeavour/policy/' + policyId,
      };
    } else {
      var applyOptionsLink = {
        rel: 'action',
        href:
          finalURLParts +
          ngp.apiFirstStep +
          'slapolicy?action=applyPolicyOptions&name=' +
          escSlapName +
          '&subtype=' +
          subType,
      };
      var optionsLink = {
        rel: 'related',
        href: finalURLParts + '/api/endeavour/policy/' + policyId,
      };
    }

    slap.links.applyOptions = applyOptionsLink;
    slap.links.options = optionsLink;

    cb();
  }
}

function getProfilesInUse(step, funcStream, ecxSessionId, oFromNg, cb) {
  const subType = ngp.getRequestState(step.reqId, ecxSessionId, 'statusSpec');
  const apps = { path: '/api/application/search?from=hlo&resourceType=database', accessor: 'contents', type: 'applicationType' };
  const vms = { path: '/api/hypervisor/search?from=hlo&resourceType=vm', accessor: 'vms', type: 'hypervisorType' };
  const resourceTypes = {
    sql: apps,
    oracle: apps,
    db2: apps,
    exch: apps,
    office365: apps,
    k8s: apps,
    file: apps,
    mongo: apps,
    openshift: apps,
    vmware: vms,
    hyperv: vms,
  };

  const resource = resourceTypes[subType];
  if (!resource) {
    ngp.setRequestState(step.reqId, ecxSessionId, 'profilesInUse', '*');
    ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, {
      statusCode: 200,
      response: null,
    });
    return;
  }

  new Promise((resolve, reject) => {
    let postBody = { name: '', [resource.type]: subType };
    ngp.sendRequestToEcx('POST', ngp.ecxExternal, resource.path, ecxSessionId, postBody,
      function (rObj) {
        if (rObj.e || rObj.response.statusCode !== 200) {
          reject(ngp.getGenericEcxErrorResponse(rObj, ecxSessionId, step));
          return;
        }
        var profileSet = new Set();
        rObj.rO[resource.accessor]
          .map((inst) => inst.storageProfiles)
          .flat()
          .forEach((p) => {
            profileSet.add(p);
          });
        resolve({ status: rObj.response.statusCode, profiles: Array.from(profileSet) });
      }
    );
  })
    .then((data) => {
      ngp.setRequestState(step.reqId, ecxSessionId, 'profilesInUse', data.profiles);
      ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, {
        statusCode: data.status,
        response: null,
      });
    })
    .catch((e) => {
      ngp.logMsg('warn', 'Failed because of error=' + e);
      ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, e, false);
    });
}
